@@ -25,7 +25,8 @@ our $LOGGERS_BY_NAME = {};
 our %APPENDER_BY_NAME = ();
 our $INITIALIZED = 0;
 our $NON_INIT_WARNED;
-
+our $DIE_DEBUG = 0;
+our $DIE_DEBUG_BUFFER = "";
     # Define the default appender that's used for formatting
     # warn/die/croak etc. messages.
 our $STRING_APP_NAME = "_l4p_warn";
@@ -852,20 +853,29 @@ sub and_warn {
 sub and_die {
 #######################################################
   my $self = shift;
-  die(callerline($self->warning_render(@_)));
+
+  my($msg) = callerline($self->warning_render(@_));
+
+  if($DIE_DEBUG) {
+      $DIE_DEBUG_BUFFER = "DIE_DEBUG: $msg";
+  } else {
+      die("$msg\n");
+  }
 }
 
 ##################################################
 sub logwarn {
 ##################################################
   my $self = shift;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   if ($self->is_warn()) {
         # Since we're one caller level off now, compensate for that.
-    $Log::Log4perl::caller_depth++;
     my @chomped = @_;
     chomp($chomped[-1]);
     $self->warn(@chomped);
-    $Log::Log4perl::caller_depth--;
     $self->and_warn(@_);
   }
 }
@@ -874,13 +884,15 @@ sub logwarn {
 sub logdie {
 ##################################################
   my $self = shift;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   if ($self->is_fatal()) {
         # Since we're one caller level off now, compensate for that.
-    $Log::Log4perl::caller_depth++;
     my @chomped = @_;
     chomp($chomped[-1]);
     $self->fatal(@chomped);
-    $Log::Log4perl::caller_depth--;
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
@@ -893,13 +905,14 @@ sub logexit {
 ##################################################
   my $self = shift;
 
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   if ($self->is_fatal()) {
         # Since we're one caller level off now, compensate for that.
-    $Log::Log4perl::caller_depth++;
     my @chomped = @_;
     chomp($chomped[-1]);
     $self->fatal(@chomped);
-    $Log::Log4perl::caller_depth--;
   }
 
   exit $Log::Log4perl::LOGEXIT_CODE;
@@ -911,16 +924,19 @@ sub logcluck {
 ##################################################
   my $self = shift;
 
-  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
+  local $Carp::CarpLevel = 
+        $Carp::CarpLevel + 1;
+
   my $msg = $self->warning_render(@_);
 
   if ($self->is_warn()) {
     my $message = Carp::longmess($msg);
-    $Log::Log4perl::caller_depth++;
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    $Log::Log4perl::caller_depth--;
     Carp::cluck($msg);
   }
 }
@@ -931,15 +947,17 @@ sub logcarp {
   my $self = shift;
 
   local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   my $msg = $self->warning_render(@_);
 
   if ($self->is_warn()) {
     my $message = Carp::shortmess($msg);
-    $Log::Log4perl::caller_depth++;
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    $Log::Log4perl::caller_depth--;
     Carp::carp($msg) if $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR;
   }
 }
@@ -951,16 +969,19 @@ sub logcroak {
 ##################################################
   my $self = shift;
 
-  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
   my $msg = $self->warning_render(@_);
 
+  local $Carp::CarpLevel = 
+        $Carp::CarpLevel + 1;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   if ($self->is_fatal()) {
     my $message = Carp::shortmess($msg);
-    $Log::Log4perl::caller_depth++;
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
     }
-    $Log::Log4perl::caller_depth--;
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
@@ -973,16 +994,19 @@ sub logconfess {
 ##################################################
   my $self = shift;
 
-  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  local $Carp::CarpLevel = 
+        $Carp::CarpLevel + 1;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   my $msg = $self->warning_render(@_);
 
   if ($self->is_fatal()) {
     my $message = Carp::longmess($msg);
-    $Log::Log4perl::caller_depth++;
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
     }
-    $Log::Log4perl::caller_depth--;
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
@@ -996,10 +1020,12 @@ sub logconfess {
 sub error_warn {
 ##################################################
   my $self = shift;
+
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   if ($self->is_error()) {
-    $Log::Log4perl::caller_depth++;
     $self->error(@_);
-    $Log::Log4perl::caller_depth--;
     $self->and_warn(@_);
   }
 }
@@ -1009,12 +1035,13 @@ sub error_die {
 ##################################################
   my $self = shift;
 
+  local $Log::Log4perl::caller_depth = 
+        $Log::Log4perl::caller_depth + 1;
+
   my $msg = $self->warning_render(@_);
 
   if ($self->is_error()) {
-    $Log::Log4perl::caller_depth++;
     $self->error($msg);
-    $Log::Log4perl::caller_depth--;
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
