@@ -3,6 +3,7 @@ use warnings;
 use strict;
 
 use File::Temp qw(tempfile);
+use File::Spec;
 
 use constant INTERNAL_DEBUG => 0;
 
@@ -42,30 +43,36 @@ sub resurrector_loader {
 ###########################################
     my ($code, $module) = @_;
 
+    print "resurrector_loader called with $module\n" if INTERNAL_DEBUG;
+
       # Skip Log4perl appenders
     if($module =~ m#^Log/Log4perl/Appender#) {
-        print "Ignoreing $module (Log4perl-internal)\n" if INTERNAL_DEBUG;
+        print "Ignoring $module (Log4perl-internal)\n" if INTERNAL_DEBUG;
         return undef;
     }
 
+    my $path = $module;
+
       # Skip unknown files
     if(!-f $module) {
           # We might have a 'use lib' statement that modified the
           # INC path, search again.
-        my $path = pm_search($module);
+        $path = pm_search($module);
         if(! defined $path) {
             print "File $module not found\n" if INTERNAL_DEBUG;
             return undef;
         }
         print "File $module found in $path\n" if INTERNAL_DEBUG;
-        $module = $path;
     }
 
-    print "Resurrecting module $module\n" if INTERNAL_DEBUG;
+    print "Resurrecting module $path\n" if INTERNAL_DEBUG;
 
-    my $fh = resurrector_fh($module);
+    my $fh = resurrector_fh($path);
+
+    my $abs_path = File::Spec->rel2abs( $path );
+    print "Setting %INC entry of $module to $abs_path\n" if INTERNAL_DEBUG;
+    $INC{$module} = $abs_path;
 
-    $INC{$module} = 1;
     return $fh;
 }
 
